
# q-expansion of newform 4851.2.a.c, downloaded from the LMFDB on 23 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 4851
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[-1], [0], [0], [0], [1], [-4], [-4], [-8], [8], [-2], [-4], [10], [-4], [4], [12], [-6], [12], [-12], [4], [0], [-4], [-8], [8], [0], [8], [-12], [4], [4], [-14], [-2], [16], [0], [-6], [-8], [10], [-8], [0], [12], [-16], [20], [-20], [24], [8], [18], [-6], [20], [20], [-28], [0], [8], [10], [0], [4], [20], [0], [16], [8], [8], [-10], [26], [32], [28], [-16], [-4], [16], [2], [4], [-2], [28], [4], [-32], [-8], [-28], [22], [20], [20], [6], [-16], [34], [-4], [28], [-10], [0], [-24], [-8], [-28], [-18], [-38], [12], [24], [-28], [24], [-24], [-12], [-28], [24], [-40], [24], [0], [-2], [-20], [18], [40], [-22], [36], [24], [-12], [36], [16], [-44], [16], [26], [10], [-4], [16], [34], [28], [4], [-18], [20], [-16], [-2], [20], [36], [20], [30], [-6], [-12], [12], [-28], [-20], [24], [-48], [10], [-12], [44], [24], [-8], [-48], [6], [-16], [10], [-32], [-12], [-32], [20], [-20], [4], [28], [-24], [-14], [-8], [52], [24], [28], [-32], [-16], [-40], [12], [12], [-28], [6], [-24], [52], [18], [12], [40], [28], [34], [-24], [-60], [32], [32], [22], [-60], [-8], [-36], [54], [24], [12], [-32], [40], [-6], [44], [-16], [-24], [-2], [-20], [-42], [-16], [-56], [-12], [4], [32], [-68], [0], [-66], [-46], [4], [-40], [50], [-32], [56], [-4], [-28], [-16], [4], [36], [-26], [36], [62], [8], [-24], [-20], [56], [32], [-40], [-28], [-48], [-34], [26], [-24], [14], [-48], [16], [10], [8], [-48], [8], [-52], [2], [-16], [40], [-42], [60], [24], [8], [-22], [20], [-60], [36], [40], [-36], [30], [-20], [-48], [60], [-36], [-28], [24], [-34], [-76], [0], [32], [-36], [-4], [58], [8], [32], [4], [0], [36], [-28], [-52], [-48], [12], [66], [-56], [44], [6], [32], [-68], [24], [16], [12], [-44], [60], [-2], [-38], [-56], [64], [0], [-8], [-4], [8], [16], [2], [-54], [4], [-52], [-30], [-28], [42], [24], [78], [-36], [-20], [-8], [-40], [-56], [28], [-82], [-56], [84], [52], [14], [-84], [-32], [-56], [70], [48], [86], [66], [-36], [80], [-20], [-76], [24], [-40], [50], [32], [38], [-68], [-24], [-26], [-48], [-36], [28], [-32], [-26], [14], [78], [0], [72], [60], [84], [50], [76], [80], [-40], [54], [-6], [52], [64], [30], [-44], [40], [-28], [16], [8], [8], [-38], [-78], [24], [86], [-36], [44], [0], [-18], [56], [38], [28], [32], [-60], [-52], [-10], [52], [24], [-22], [-52], [4], [72], [74], [-32], [46], [-76], [48], [-28], [60], [-28], [-48], [54], [-48], [30], [56], [76], [-24], [-16], [-36], [-68], [62], [-56], [36], [76], [-8], [-58], [40], [-48], [-100], [54], [-44], [-2], [-8], [-80], [100], [16], [-82], [94], [100], [-36], [-20], [86], [-84], [52], [22], [0], [-72], [76], [96], [-16], [-2], [-92], [24], [96], [-48], [-24], [-12], [90], [-44], [8]]
